' ------------------------------------------------------------------------------
' ILPER 1.35 for Windows
' Copyright (c) 2008-2009  J-F Garnier
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
' 
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
' 
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' ----------------------------------------------------------------------------------
'
' MainForm.vb  Main module, GUI, PIL-Box management
'
' Oct  2008: standalone ILPER Windows version using the PILBox!
' May  2009: Version 1.1 released as free software
' June 2009: Version 1.2 correction and improvements for PILBox 1.2:
'            correct COFF frame, disable the PILBox at ILPer exit 
' July 2009: Version 1.3 use 8-bit serial format 
' Oct  2009: Version 1.31 bug fix in ildrive, minor improvements
' Nov  2009: Version 1.33 changed display to printer device type in ildisp, minor improvements
' Nov  2009: Version 1.34 fixed a minor bug in ilper7 (scope function)
' Nov  2009: Version 1.35 minor GUI changes
' ---------------------------------------------------------------------------------


' --------------------------------
' MainForm
' Main application form for ILPer
' --------------------------------
Public Class MainForm

    Dim frun As Boolean = False
    Dim lasth As Integer = 0      ' store the last sent/received high byte on serial
    Dim fesc As Integer = 0       ' escape sequence

  
    
   ' PIL-Box special init command frames:
    Const CON = &H496     ' CON
    Const COFF = &H497    ' COFF
    Const TDIS = &H494    ' TDIS

    Const USE8BITS = True ' use 8-bit format with PILBox

    ' --------------------------------
    ' DisplayStr()
    ' display a string in the display box
    ' manage the HP-41C specific characters
    ' --------------------------------
    Public Sub DisplayStr(ByVal ch As String)
        If Len(ch) = 1 Then
            If fesc = 0 Then
                Select Case (Asc(ch))
                    ' convert special HP41 characters to regular ASCII 
                    Case 0 : ch = "*"
                    Case 12 : ch = "u"   ' micron
                    Case 28 : ch = "s"   ' sigma
                    Case 27 : fesc = 1   ' escape sequences
                    Case 29 : ch = "#"   ' different
                    Case 124 : ch = "a"  ' angle sign
                    Case 127 : ch = "`"  ' append
                    Case Is > 127 : ch = "."  ' non-printable characters
                End Select
                If fesc = 0 Then TextBox_display.AppendText(ch)
            Else
                fesc = 0  ' ignore escape sequences (for the HP71)
            End If
        End If
    End Sub

    ' --------------------------------
    ' ClrDisplay()
    ' clear the display box
    ' --------------------------------
    Public Sub ClrDisplay()
        TextBox_display.Clear()
    End Sub

    ' --------------------------------
    ' DisplayMnemo()
    ' display a string in the scope box
    ' (if the scope is enabled)
    ' --------------------------------
    Public Sub DisplayMnemo(ByVal ch As String)
        Static cpt As Integer = 0
        If CheckBox_scope.Checked Then
            TextBox_scope.AppendText(ch)
            cpt = cpt + 1
            If cpt = 3 Then
                cpt = 0
                TextBox_scope.AppendText(Chr(13) & Chr(10))
            End If
        End If
    End Sub



    ' --------------------------------
    ' InitPILBox()
    ' init the PIL Box 
    ' --------------------------------
    Private Function InitPILBox(ByVal cmd As Integer) As Integer
        Dim byt As Integer

        lasth = 0
        SendFrame(cmd)
        Try
            byt = SerialPort1.ReadByte()
        Catch
            byt = 0
        End Try
        If (byt And &H3F) = (cmd And &H3F) Then
            InitPILBox = 0   ' ok
        Else
            InitPILBox = -1  ' error
        End If

    End Function

    ' --------------------------------
    ' SendFrame()
    ' send a IL frame to the PILBox
    ' --------------------------------
    Private Sub SendFrame(ByVal frame As Integer)
        Dim lbyt As Integer, hbyt As Integer  ' low/high bytes to serial
        Dim buf(2) As Byte
        Dim n As Integer

        ' build the low and high parts
        If Not USE8BITS Then
            ' use 7-bit characters for maximum compatibility
            hbyt = ((frame >> 6) And &H1F) Or &H20
            lbyt = (frame And &H3F) Or &H40
        Else
            ' 8-bit format for optimum speed
            hbyt = ((frame >> 6) And &H1E) Or &H20
            lbyt = (frame And &H7F) Or &H80
        End If
        If (hbyt <> lasth) Then
            ' send high part if different from last one
            lasth = hbyt
            buf(1) = hbyt : buf(2) = lbyt : n = 2
        Else
            ' otherwise send only low part
            buf(1) = lbyt : n = 1
        End If
        SerialPort1.Write(buf, 1, n)

    End Sub


    ' --------------------------------
    ' PILBox()
    ' manage the PILBox
    ' (based on Emu41 ilext2 module)
    ' called at each received byte
    ' --------------------------------
    Private Sub PILBox(ByVal byt As Integer)
        Dim frame As Integer     ' IL frame

        If ((byt And &HC0) = 0) Then
            If (byt And &H20) Then
                ' high byte, save it
                lasth = byt
                SerialPort1.Write(Chr(13))   ' acknowledge
            End If
        Else
            ' low byte, build frame according to format 
            If (byt And &H80) Then
                frame = ((lasth And &H1E) << 6) + (byt And &H7F)
            Else
                frame = ((lasth And &H1F) << 6) + (byt And &H3F)
            End If

            frame = hpil_transmit(frame)  ' transmit IL frame to internal virtual devices

            ' send returned frame to PILBox
            SendFrame(frame)

        End If

    End Sub


    ' --------------------------------
    ' START button - main loop
    ' --------------------------------
    Private Sub ButtonStart_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles ButtonStart.Click
        Dim byt As Integer

        ClrDisplay()

        SerialPort1.PortName = TextBox_ComPort.Text
        If RadioButton_highspeed.Checked Then SerialPort1.BaudRate = 115200 Else SerialPort1.BaudRate = 9600

        Try
            SerialPort1.Open()
            byt = 1
        Catch
            byt = 0
        End Try
        If byt = 0 Then
            MessageBox.Show("Error: COM port not available.", "My Application", _
                       MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Exit Sub
        End If

        '        My.Application.DoEvents()

        ' init the PIL-Box in control-off mode 
        If InitPILBox(COFF) < 0 Then
            MessageBox.Show("Error: no response from PILBox.", "My Application", _
                       MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            SerialPort1.Close()
            Exit Sub
        End If

        ButtonStop.Enabled = True
        ButtonStart.Enabled = False
        TextBox_filename.Enabled = False
        TextBox_ComPort.Enabled = False
        RadioButton_lowspeed.Enabled = False
        RadioButton_highspeed.Enabled = False

        frun = True
        init_hpil()

        Do

            Try
                byt = SerialPort1.ReadByte()
            Catch
                byt = 0
            End Try
            If byt <> 0 Then
                PILBox(byt)
            End If

            My.Application.DoEvents()

        Loop While frun


        Try
            InitPILBox(TDIS)   ' stop the PIL-Box       
        Catch
            ' in case the PILBox is disconnected before closing ILPer...
        End Try
        Try
            SerialPort1.Close()
        Catch
            ' in case the PILBox is disconnected before closing ILPer...
        End Try

    End Sub





    ' --------------------------------
    ' STOP button
    ' --------------------------------
    Private Sub ButtonStop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonStop.Click
        ButtonStop.Enabled = False
        ButtonStart.Enabled = True
        TextBox_filename.Enabled = True
        TextBox_ComPort.Enabled = True
        RadioButton_lowspeed.Enabled = True
        RadioButton_highspeed.Enabled = True
        frun = False
    End Sub

    ' --------------------------------
    ' Form closing
    ' --------------------------------
    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        frun = False
    End Sub


   
End Class
