' ------------------------------------------------------------------------------
' ILPER 1.33 for Windows
' Copyright (c) 2008-2009  J-F Garnier
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
' 
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
' 
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' ------------------------------------------------------------------------------
'                                            
' ildisp.vb   HP-IL display module           
' (used in Emu41/Emu71/ILper)                
'                                            
' Based on previous personal work:           
' 1986: ILPER4 video+disc, (6502 assembler)  
' 1988: ILPER5 videoil module                
' 1993: ILPER ported on PC (8086 assembler)  
' 1997: rewriten in C and included in Emu41  
' 2008: rewriten in VB for the standalone ILPER Windows version using the PILBox!
' oct 2009: changed the AID to 46 (for general 'dumb' printer)
' ------------------------------------------------------------------------------

Module ildisp

    ' HP-IL data and variables */
    Private Const AID = 46     ' Accessory ID = printer
    Private Const DEFADDR = 3  ' default address after AAU
    Private did As String = "DISPLAY" & Chr(13)  ' Device ID 
    Private status As Integer  ' HP-IL status (always 0 here)
    Private adr As Integer     ' bits 0-5 = HP-IL address, bit 7 = 1 means auto address taken 
    Private fvideo As Integer  ' HP-IL state machine flags: 
    ' bit 7, bit 6:
    ' 0      0       idle
    ' 1      0       addressed listen
    ' 0      1       addressed talker 
    ' 1      1       active talker
    ' ( this choice comes from my original 6502 assembly code [PLTERx and ILPERx applications, 1984-1988]
    '   that used the efficient BIT opcode to test both bit 6 and 7)
    Private ptsdi As Integer  ' output pointer for device ID 

    ' ****************************************** 
    ' init_ildisplay()                           
    '                                            
    ' init the virtual HPIL display device       
    ' ****************************************** 
    Sub init_ildisplay()
        status = 0
        adr = 0
        fvideo = 0
        ptsdi = 0
    End Sub



    ' ****************************************** 
    ' traite_doe(frame)                          
    '                                            
    ' manage the HPIL data frames                
    ' returns the returned frame                 
    ' ****************************************** 
    Private Function traite_doe(ByVal frame As Integer) As Integer

        Dim n As Integer
        Dim c As Char

        n = frame And 255
        If (fvideo And &H80) Then
            ' addressed 
            If (fvideo And &H40) Then
                ' talker 
                If (ptsdi > 0) Then
                    c = Mid(did, ptsdi, 1)
                    frame = Asc(c)
                    If Asc(c) = 13 Then ptsdi = 0 Else ptsdi = ptsdi + 1
                End If
                If (ptsdi = 0) Then
                    frame = &H540  ' EOT 
                    fvideo = &H40
                End If
            Else
                ' listener 
                MainForm.DisplayStr(Chr(n))
            End If
        End If
        Return (frame)
    End Function



    ' ****************************************** 
    ' traite_cmd(frame)                          
    '                                            
    ' manage the HPIL command frames             
    ' returns the returned frame                 
    ' ****************************************** 
    Private Function traite_cmd(ByVal frame As Integer) As Integer

        Dim n As Integer

        n = frame And 255
        Select Case (n >> 5)
            Case 0
                Select Case (n)
                    Case 4 ' SDC 
                        If (fvideo And &H80) Then MainForm.ClrDisplay()
                    Case 20 ' DCL 
                        MainForm.ClrDisplay()
                End Select
            Case 1 'LAD 
                n = n And 31
                If (n = 31) Then
                    ' UNL, if not talker then go to idle state 
                    If ((fvideo And &H40) = 0) Then fvideo = 0
                Else
                    ' else, if MLA go to listen state 
                    If (n = (adr And 31)) Then fvideo = &H80
                End If
            Case 2 ' TAD 
                n = n And 31
                If (n = (adr And 31)) Then
                    ' if MTA go to talker state 
                    fvideo = &H40
                Else
                    ' else if addressed talker, go to idle state 
                    If ((fvideo And &H40) <> 0) Then fvideo = 0
                End If
            Case 4
                n = n And 31
                Select Case (n)
                    Case 16 ' IFC 
                        fvideo = 0
                    Case 26 ' AAU 
                        adr = DEFADDR
                End Select
        End Select

        traite_cmd = frame

    End Function




    ' ******************************************
    ' traite_rdy(frame)                          
    '                                            
    ' manage the HPIL ready frames               
    ' returns the returned frame                 
    ' ****************************************** 
    Private Function traite_rdy(ByVal frame As Integer) As Integer
        Dim n As Integer

        n = frame And 255
        If (n <= 127) Then
            ' sot 
            If (fvideo And &H40) Then
                ' if addressed talker 
                If (n = 66) Then   ' NRD 
                    ptsdi = 0     ' abort transfer 
                ElseIf (n = 96) Then  ' SDA 
                    ' no response 
                ElseIf (n = 97) Then ' SST 
                    frame = status
                    fvideo = &HC0  ' active talker
                ElseIf (n = 98) Then ' SDI 
                    frame = Asc(Left(did, 1))
                    ptsdi = 2
                    fvideo = &HC0
                ElseIf (n = 99) Then ' SAI 
                    frame = AID
                    fvideo = &HC0
                End If
            End If
        ElseIf (n < &H9E) Then
            ' AAD, if not already an assigned address, take it 
            If ((adr And &H80) = 0) Then
                adr = n
                frame = frame + 1
            End If
        End If

        traite_rdy = frame


    End Function



    ' ****************************************** 
    ' ildisplay(frame)                           
    '                                            
    ' manage a HPIL frame                       
    ' returns the returned frame                 
    ' ****************************************** 
    Function ILdisplay(ByVal frame As Integer) As Integer
        If ((frame And &H400) = 0) Then
            frame = traite_doe(frame)
        ElseIf ((frame And &H700) = &H400) Then
            frame = traite_cmd(frame)
        ElseIf ((frame And &H700) = &H500) Then
            frame = traite_rdy(frame)
        End If
        ILdisplay = frame
    End Function

End Module
