' ------------------------------------------------------------------------------
' ILPER 1.33 for Windows
' Copyright (c) 2008-2009  J-F Garnier
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
' 
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
' 
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' ------------------------------------------------------------------------------
'                                            
' ildrive.vb    Disk drive module          
' (used in Emu41/Emu71/ILper)                
'                                            
' Based on previous personal work:           
' 1986: ILPER4 video+disc, (6502 assembler)  
' 1988: ILPER5 discil module                
' 1993: ILPER ported on PC (8086 assembler)  
' 1997: rewriten in C and included in Emu41  
' 2008: rewriten in VB for the standalone ILPER Windows version using the PILBox!
' Oct 2009: bug fix in copybuf() and exchbuf() (VB issue...)
' ------------------------------------------------------------------------------

Imports system.IO


Module ildrive

    ' HP-IL data and variables */
    Private Const AID = 16     ' Accessory ID = mass storage
    Private Const DEFADDR = 2  ' default address after AAU
    Private did As String = "HDRIVE1" & Chr(13)  ' Device ID 
    Private status As Integer  ' HP-IL status 
    Private adr As Integer     ' bits 0-5 = HP-IL address, bit 7 = 1 means auto address taken 
    Private fetat As Integer  ' HP-IL state machine flags: 
    ' bit 7, bit 6:
    ' 0      0       idle
    ' 1      0       addressed listen
    ' 0      1       addressed talker 
    ' 1      1       active talker
    ' bit 1: SDA, SDI
    ' bit 0: SST, SDI, SAI 
    Private ptsdi As Integer  ' output pointer for device ID 

    ' disk management variables  
    Private devl As Integer, devt As Integer     ' dev lstn & tlker
    Private oc As Integer             ' byte pointer
    Private pe As Integer             ' record pointer */
    Private pe0 As Integer
    Private fpt As Integer            ' flag pointer 
    Private flpwr As Integer          ' flag partial write 
    Private ptout As Integer          ' pointer out 
    ' 640kb floppy drive descriptor 
    Private lif As Byte() = New Byte(11) {0, 0, 0, 80, 0, 0, 0, 2, 0, 0, 0, 16}   ' 80 tracks, 2 sides, 16 sect/track */
    Private nbe As Integer = 2560  ' number of 256-byte sectors 

    ' 128Kb cassette drive descriptor would be:
    ' lif[] = {0, 0, 0, 2, 0, 0, 0, 1, 0, 0, 1, 0 } ; /* 2 tracks, 1 side, 256 sect/track */
    ' nbe = 512;   /* number of 256-byte sectors */

    Private buf0(255) As Byte   ' buffer 0 
    Private buf1(255) As Byte   ' buffer 1 
    Private hdiscfile As String   ' path to disk image file 


    ' ***************************************************************** 
    ' hardware access functions                                         
    ' ***************************************************************** 


    ' read one sector n pe (256 bytes) into buf0 
    Private Sub rrec()

        Dim n As Integer

        If File.Exists(hdiscfile) Then

            Dim fs As New FileStream(hdiscfile, FileMode.Open, FileAccess.Read)
            Try
                fs.Seek(pe * 256, SeekOrigin.Begin)
                fs.Read(buf0, 0, 256)
            Catch
                For n = 0 To 255 : buf0(n) = 255 : Next n
            End Try
            fs.Close()
        Else
            status = 20
        End If


    End Sub

    ' write buf0 to one sector n pe (256 bytes) 
    Private Sub wrec()

        If File.Exists(hdiscfile) Then

            Dim fs As New FileStream(hdiscfile, FileMode.Open, FileAccess.ReadWrite)
            Try
                fs.Seek(pe * 256, SeekOrigin.Begin)
                fs.Write(buf0, 0, 256)
            Catch
                status = 24
            End Try
            fs.Close()
        Else
            status = 29
        End If


    End Sub


    ' "format" a LIF image file 
    Private Sub format_disc()

        Dim buf2(256) As Byte
        Dim n As Integer

        status = 0
        For n = 0 To 255 : buf2(n) = 255 : Next n

        Dim fs As New FileStream(hdiscfile, FileMode.Create)
        Try
            fs.Seek(0, SeekOrigin.Begin)
            For n = 0 To 127 : fs.Write(buf2, 0, 256) : Next n
        Catch
            status = 29
        End Try

        fs.Close()

    End Sub




    ' ****************************************** 
    ' clrdrv()                                   
    '                                            
    ' Clear drive, reset internal pointers       
    ' ******************************************
    Private Sub clrdrv()
        fpt = 0  ' reset DDL4 partial sequence 
        pe = 0   ' record pointer = 0 
        oc = 0   ' byte pointer = 0 
    End Sub


    ' ****************************************** 
    ' indata(n)                                  
    '                                            
    ' receive data to disc according to DDL cmd  
    ' ****************************************** 
    Private Sub indata(ByVal n As Integer)
        Select Case (devl)
            Case 0, 2, 6
                buf0(oc) = n And 255
                oc = oc + 1
                If (oc > 255) Then
                    oc = 0
                    wrec()
                    pe = pe + 1
                    If (flpwr <> 0) Then rrec()
                Else
                    If (n And &H200) Then
                        ' END 
                        wrec()
                        If (flpwr = 0) Then pe = pe + 1
                    End If
                End If
            Case 1
                buf1(oc) = n And 255
                oc = oc + 1
                If (oc > 255) Then oc = 0
            Case 3
                oc = n And 255
            Case 4
                n = n And 255
                If (fpt <> 0) Then
                    pe0 = pe0 And &HFF00
                    pe0 = pe0 Or n
                    If (pe0 < nbe) Then
                        pe = pe0
                        status = 0
                    Else
                        status = 28
                    End If
                    fpt = 0
                Else
                    pe0 = pe0 And 255
                    pe0 = pe0 Or (n << 8)
                    fpt = fpt - 1
                End If

        End Select

    End Sub


    ' ****************************************** 
    ' outdta()                                   
    '                                            
    ' send data from disc according to DDT cmd   
    ' ****************************************** 
    Private Function outdta() As Integer

        Dim frame As Integer

        Select Case (devt)
            Case 0, 2
                frame = buf0(oc)
                oc = oc + 1
                If (oc > 255) Then
                    oc = 0
                    rrec()
                    pe = pe + 1
                End If
            Case 1
                frame = buf1(oc)
                oc = oc + 1
                If (oc > 255) Then
                    oc = 0
                    devt = 15
                End If
            Case 3
                Select Case (ptout)
                    Case 0 : frame = pe >> 8
                    Case 1 : frame = pe And 255
                    Case 2 : frame = oc And 255
                    Case Else : frame = &H540  ' EOT 
                End Select
                ptout = ptout + 1
            Case 6
                If (ptout < 12) Then
                    frame = lif(ptout)
                    ptout = ptout + 1
                Else
                    frame = &H540  ' EOT 
                End If
            Case 7
                Select Case (ptout)
                    Case 0 : frame = nbe >> 8
                    Case 1 : frame = nbe And 255
                    Case Else : frame = &H540   ' EOT 
                End Select
                ptout = ptout + 1
            Case Else : frame = &H540 ' EOT 
        End Select
        If (frame = &H540) Then   ' EOT
            fetat = &H40   ' JFG 30/11/03
        End If
        outdta = frame
    End Function





    ' ****************************************** 
    ' traite_doe(frame)                          
    '                                            
    ' manage the HPIL data frames                
    ' returns the returned frame                 
    ' ****************************************** 
    Private Function traite_doe(ByVal frame As Integer) As Integer

        Dim c As Char

        If (fetat And &H80) Then
            ' addressed 
            If (fetat And &H40) Then
                ' talker 
                If (fetat And 2) Then
                    ' data (SDA) or accessory ID (SDI) 
                    If (fetat And 1) Then
                        ' SDI 
                        If (ptsdi) Then
                            c = Mid(did, ptsdi, 1)
                            If (c = Chr(13)) Then
                                ptsdi = 0
                            Else
                                ptsdi = ptsdi + 1
                                frame = Asc(c)
                            End If
                        End If
                        If (ptsdi = 0) Then
                            frame = &H540 ' EOT 
                            fetat = &H40
                        End If
                    Else
                        ' SDA 
                        frame = outdta()
                    End If
                Else
                    ' end of SST, SAI 
                    frame = &H540  ' EOT 
                    fetat = &H40
                End If
            Else
                ' listener 
                indata(frame)
            End If
        End If
        traite_doe = frame
    End Function



    ' ****************************************** 
    ' copybuf()                                  
    '                                            
    ' copy buffer 0 to buffer 1                  
    ' ****************************************** 
    Private Sub copybuf()
        Dim n As Integer
        oc = 0
        ' buf1 = buf0  ' this doesn't work...
        For n = 0 To 255 : buf1(n) = buf0(n) : Next n  ' bug fix 2oct09
    End Sub

    ' ****************************************** 
    ' exchbuf()                                  
    '                                            
    ' exchange buffers 0 and 1                   
    ' ****************************************** 
    Private Sub exchbuf()
        Dim n As Integer
        Dim buf2(256) As Byte

        oc = 0
        ' buf2 = buf1
        ' buf1 = buf0
        ' buf0 = buf2
        For n = 0 To 255 : buf2(n) = buf1(n) : Next n ' bug fix 2oct09
        For n = 0 To 255 : buf1(n) = buf0(n) : Next n
        For n = 0 To 255 : buf0(n) = buf2(n) : Next n

    End Sub


    ' ****************************************** 
    ' traite_cmd(frame)                          
    '                                            
    ' manage the HPIL command frames             
    ' returns the returned frame                 
    ' ****************************************** 
    Private Function traite_cmd(ByVal frame As Integer) As Integer

        Dim n As Integer

        n = frame And 255
        Select Case (n >> 5)
            Case 0
                Select Case (n)
                    Case 4 ' SDC 
                        If (fetat And &H80) Then clrdrv()
                    Case 20 ' DCL 
                        clrdrv()
                End Select
            Case 1 'LAD 
                n = n And 31
                If (n = 31) Then
                    ' UNL, if not talker then go to idle state 
                    If ((fetat And &H40) = 0) Then fetat = 0
                Else
                    ' else, if MLA go to listen state 
                    If (n = (adr And 31)) Then fetat = &H80
                End If
            Case 2 ' TAD 
                n = n And 31
                If (n = (adr And 31)) Then
                    ' if MTA go to talker state 
                    fetat = &H40
                Else
                    ' else if addressed talker, go to idle state 
                    If ((fetat And &H40) <> 0) Then fetat = 0
                End If
            Case 4
                n = n And 31
                Select Case (n)
                    Case 16 ' IFC 
                        fetat = 0
                    Case 26 ' AAU 
                        adr = DEFADDR
                End Select
            Case 5 ' DDL 
                n = n And 31
                If ((fetat And &HC0) = &H80) Then
                    devl = n
                    Select Case (n)
                        Case 1 : flpwr = 0
                        Case 2 : oc = 0 : flpwr = 0
                        Case 4 : flpwr = 0 : fpt = 0 ' 01/02/03 
                        Case 5 : format_disc()
                        Case 6 : flpwr = &H80 : rrec()
                        Case 7 : fpt = 0 : pe = 0 : oc = 0
                        Case 8 : wrec() : If (flpwr = 0) Then pe = pe + 1
                        Case 9 : copybuf()
                        Case 10 : exchbuf()
                    End Select
                End If
            Case 6 ' DDT 
                n = n And 31
                If ((fetat And &H40) = &H40) Then
                    devt = n
                    ptout = 0
                    Select Case (n)
                        Case 0 : flpwr = 0
                        Case 2 : rrec() : oc = 0 : flpwr = 0 : pe = pe + 1
                        Case 4 : exchbuf()
                    End Select
                End If
        End Select

        traite_cmd = frame

    End Function




    ' ******************************************
    ' traite_rdy(frame)                          
    '                                            
    ' manage the HPIL ready frames               
    ' returns the returned frame                 
    ' ****************************************** 
    Private Function traite_rdy(ByVal frame As Integer) As Integer
        Dim n As Integer

        n = frame And 255
        If (n <= 127) Then
            ' sot 
            If (fetat And &H40) Then
                ' if addressed talker 
                If (n = 66) Then   ' NRD 
                    fetat = fetat And &HFD    ' abort transfer 
                ElseIf (n = 96) Then  ' SDA 
                    fetat = &HC2   ' moved before outdta JFG 30/11/03 
                    frame = outdta()
                ElseIf (n = 97) Then ' SST 
                    frame = status
                    status = status And &HE0
                    fetat = &HC1   ' active talker (single byte status) 
                ElseIf (n = 98) Then ' SDI 
                    frame = Asc(Left(did, 1))
                    ptsdi = 2
                    fetat = &HC3  ' active talker (multiple byte status) 
                ElseIf (n = 99) Then ' SAI 
                    frame = AID
                    fetat = &HC1   ' active talker (single byte status) 
                End If
            End If
        ElseIf (n < &H9E) Then
            ' AAD, if not already an assigned address, take it 
            If ((adr And &H80) = 0) Then
                adr = n
                frame = frame + 1
            End If
        End If

        traite_rdy = frame

    End Function

    ' ****************************************** 
    ' init_ildisplay()                           
    '                                            
    ' init the virtual HPIL display device       
    ' ****************************************** 
    Sub init_ilhdisc(ByVal filename As String)
        If filename <> "" Then hdiscfile = filename Else hdiscfile = "hdrive1.dat"
        status = 0
        adr = 0
        fetat = 0
        ptsdi = 0
    End Sub




    ' ****************************************** 
    ' ildisplay(frame)                           
    '                                            
    ' manage a HPIL frame                       
    ' returns the returned frame                 
    ' ****************************************** 
    Function ILhdisc(ByVal frame As Integer) As Integer
        If ((frame And &H400) = 0) Then
            frame = traite_doe(frame)
        ElseIf ((frame And &H700) = &H400) Then
            frame = traite_cmd(frame)
        ElseIf ((frame And &H700) = &H500) Then
            frame = traite_rdy(frame)
        End If
        ILhdisc = frame
    End Function

End Module
